
#import "ApertureStatisticsExportPlugIn.h"

@implementation ApertureStatisticsExportPlugIn

-(id)initWithAPIManager:(id<PROAPIAccessing>)anApiManager
{
    
    if ((self = [super initWithNibName:@"ApertureStatisticsExportPlugIn" bundle:[NSBundle bundleForClass:[self class]]])) {
		self.apiManager	= anApiManager;
		self.exportManager = [self.apiManager apiForProtocol:@protocol(ApertureExportManager)];
        
		if (self.exportManager == nil)
			return nil;
		
        self.mdCollector = [[[MetaDataCollector alloc] init] autorelease];
        
		self.progressLock = [[[NSLock alloc] init] autorelease];
        
        NSDictionary *persistantDomain = [[NSUserDefaults standardUserDefaults] persistentDomainForName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]];
        
        self.lastPath = [persistantDomain valueForKey:kLastPathUserDefaultsKey];
    }
    
	return self;
}

@synthesize apiManager;
@synthesize exportManager;
@synthesize progressLock;
@synthesize mdCollector;

@synthesize firstView;
@synthesize lastView;

@synthesize lastPath;

-(NSView *)settingsView
{
    return self.view;
}

-(void)willBeActivated
{
    // Nothing needed here
}

-(void)willBeDeactivated
{
    // Nothing needed here
}

-(BOOL)allowsOnlyPlugInPresets
{
    return NO;
}

-(BOOL)allowsMasterExport
{
    return NO;
}

-(BOOL)allowsVersionExport
{
    return YES;
}

-(BOOL)wantsFileNamingControls
{
    return NO;
}

-(void)exportManagerExportTypeDidChange
{
    // No masters so it should never get this call.
}

- (BOOL)wantsDestinationPathPrompt
{
    return YES;
}

- (NSString *)defaultDirectory
{
    return [self.lastPath stringByExpandingTildeInPath];
}

- (NSString *)destinationPath
{
    return nil;
}

- (void)exportManagerShouldBeginExport
{
    @synchronized(exportManager) {
        [self.exportManager shouldBeginExport];
    }
}

- (void)exportManagerWillBeginExportToPath:(NSString *)path
{
    // Save our export base path to use later.
	self.lastPath = path;
    
    
}

- (BOOL)exportManagerShouldExportImageAtIndex:(unsigned)index
{
    // "Export" all images
    return YES;
}

- (void)exportManagerWillExportImageAtIndex:(unsigned)index
{
    // Nothing to confirm here.
}

- (BOOL)exportManagerShouldWriteImageData:(NSData*)imageData
                           toRelativePath:(NSString*)path
                          forImageAtIndex:(unsigned)index
{
    // Update the progress
	[self lockProgress];
	[exportProgress.message release];
	exportProgress.message = [NSLocalizedStringFromTableInBundle(@"exporting", @"Localizable", [NSBundle bundleForClass:[self class]], @"Exporting...") retain];
	exportProgress.currentValue = index + 1;
	[self unlockProgress];
    
    NSLog(@"Exporting image %i to %@", index, path);
    
    /*
    NSBitmapImageRep *imageTest = [NSBitmapImageRep imageRepsWithData:imageData];
    NSDictionary *exifData = [imageTest valueForProperty:@"NSImageEXIFData"];
    
    NSLog(@"Exif Data: %@",[imageTest valueForProperty:@"NSImageEXIFData"]) ;
    */
    
    CFDataRef cfdata = CFDataCreate(NULL, [imageData bytes], [imageData length]);
    CGImageSourceRef imageSourceRef = CGImageSourceCreateWithData(cfdata, nil);
    CFDictionaryRef imageProperties = CGImageSourceCopyPropertiesAtIndex(imageSourceRef, 0, nil);
    
    if (imageProperties) {
        /*
        NSNumber *width = (NSNumber *)CFDictionaryGetValue(imageProperties, kCGImagePropertyPixelWidth);
        NSNumber *height = (NSNumber *)CFDictionaryGetValue(imageProperties, kCGImagePropertyPixelHeight);
        NSLog(@"Image dimensions: %@ x %@ px", width, height);
        */
        
        CFDictionaryRef exif = CFDictionaryGetValue(imageProperties, kCGImagePropertyExifDictionary);
        if (exif) {
            /*
            NSString *dateTakenString = (NSString *)CFDictionaryGetValue(exif, kCGImagePropertyExifDateTimeOriginal);
            NSLog(@"Date Taken: %@", dateTakenString);
            */
            
            NSString *fNumber = (NSString *)CFDictionaryGetValue(exif, kCGImagePropertyExifFNumber);
            [self.mdCollector addApertureWithValue:fNumber];
            
            NSLog(@"Aperture: %@", fNumber);
        }
        
        CFRelease(imageProperties);
    }
    
    // Tell Aperture to avoid an image export
    return NO;
}

- (void)exportManagerDidWriteImageDataToRelativePath:(NSString *)path
								     forImageAtIndex:(unsigned)index
{
    // Nothing to do here
}

- (void)exportManagerDidFinishExport
{
    // TODO: Creation of result image(s)
    
    StatisticsImageCreator *imageCreator = [[StatisticsImageCreator alloc] initWithImageMetaData:self.mdCollector];
    
    [imageCreator createImageAtPath:self.lastPath];
    
    // Cleanup!
    
    @synchronized(exportManager) {
        [self.exportManager shouldFinishExport];
    }
}

- (void)exportManagerShouldCancelExport
{
    // TODO: Delete files if required
    
    @synchronized(exportManager) {
        [self.exportManager shouldCancelExport];
    }
}

- (ApertureExportProgress *)progress
{
    return &exportProgress;
}

- (void)lockProgress {
	[self.progressLock lock];
}

- (void)unlockProgress {
	[self.progressLock unlock];
}

- (void)dealloc
{
    
    self.mdCollector = nil;
    
    self.lastPath = nil;
    
    self.apiManager = nil;
    self.exportManager = nil;
    self.progressLock = nil;
    
    [super dealloc];
}

@end
